//		Copyright © 1997	Jens Kilian
//
//		This program is free software; you can redistribute it and/or modify
//		it under the terms of the GNU General Public License as published by
//		the Free Software Foundation; either version 1, or (at your option)
//		any later version.
//
//		This program is distributed in the hope that it will be useful,
//		but WITHOUT ANY WARRANTY; without even the implied warranty of
//		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//		GNU General Public License for more details.
//
//		You should have received a copy of the GNU General Public License
//		along with this program; if not, write to the Free Software
//		Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//		Screen saver thread; controls when knots are drawn.

#include <cstdlib>
#include <Bitmap.h>
#include <GraphicsDefs.h>
#include <Point.h>
#include <Rect.h>
#include <View.h>
#include <Window.h>

#include "CSaveThread.h"
#include "KnotThread.h"
#include "RandomKnot.h"
#include "ClosedKnot.h"
#include "HorizontalKnot.h"
#include "VerticalKnot.h"
#include "beoslogo.h"

const rgb_color kBlack = { 0, 0, 0, 0 };

KnotThread::KnotThread(void)
:	mKnotsDrawn(0),
	mLogoCounter(0),
	mLogo(BRect(B_ORIGIN, BPoint(logo_width-1, logo_height-1)), B_COLOR_8_BIT)
{
	mLogo.SetBits(logo_bits, sizeof(logo_bits), 0, B_COLOR_8_BIT);
}

KnotThread::~KnotThread(void)
{
	// empty
}

void
KnotThread::StartSaving(BView *pView)
{
	mKnotsDrawn = 5;
	mLogoCounter = 0;
	inherited::StartSaving(pView);	
}
	
void
KnotThread::StopSaving(void)
{
	inherited::StopSaving();
}

double
KnotThread::Save(void)
{
	if (mView->Window()->Lock()) {
		const BRect bounds = mView->Bounds();
		
		if (++mKnotsDrawn > 4 || rand() % 10 > 7) {
			mView->SetHighColor(kBlack);
			mView->FillRect(bounds);
			mKnotsDrawn = 0;
		}

		if ((mLogoCounter += rand() % 3) > 50) {
			// Let's hear it for the BeOS!
			const int xPos = rand() % (bounds.IntegerWidth() - logo_width);
			const int yPos = rand() % (bounds.IntegerHeight() - logo_height);
			mView->DrawBitmap(&mLogo, BPoint(xPos, yPos));
			mLogoCounter = 0;
		
		} else {
			// Draw any old knot.
			bool drawn;
			switch (rand() % 5) {

			case 0:
				do {
					HorizontalKnot knot(mView);
					drawn = knot.Draw();
				} while (!drawn);
				break;
			
			case 1:
				do {
					VerticalKnot knot(mView);
					drawn = knot.Draw();
				} while (!drawn);
				break;
			
			default:
				do {
					ClosedKnot knot(mView);
					drawn = knot.Draw();
				} while (!drawn);
				break;
			}
		}				
		mView->Window()->Unlock();
	}
	
	return 1000000.0 * (double)(rand() % 5 + 2);
}
